/*
 *	Sherlock Filter Engine
 *
 *	(c) 1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

struct filter_cond {
  int op;
  union {
    struct {
      struct filter_cond *l, *r;
    } expr;
    struct {
      uns what;
      void *patt;
    } match;
    struct {
      uns what, num;
    } cmp;
  } u;
};

#define F_C_AND 0
#define F_C_OR 1
#define F_C_NOT 2
#define F_C_MATCH_REGEX 3
#define F_C_MATCH_PATTERN 4
#define F_C_MATCH_PATTERN_NOCASE 5
#define F_C_MATCH_EQUAL 6
#define F_C_MATCH_NUMBER_EQ 8	/* Numeric conditions: cond^1 gives negated version */
#define F_C_MATCH_NUMBER_NE 9
#define F_C_MATCH_NUMBER_GT 10
#define F_C_MATCH_NUMBER_LE 11
#define F_C_MATCH_NUMBER_GE 12
#define F_C_MATCH_NUMBER_LT 13

#define F_VF_NUMERIC 0x1000
#define F_VF_READONLY 0x2000

#define F_VAR_URL (F_VF_READONLY | 0)
#define F_VAR_HOST (F_VF_READONLY | 1)
#define F_VAR_SOURCE (F_VF_READONLY | 2)
#define F_VAR_PORT (F_VF_READONLY | F_VF_NUMERIC | 3)
#define F_VAR_TAG (F_VF_NUMERIC | 4)
#define F_VAR_WEIGHT (F_VF_NUMERIC | 5)
#define F_VAR_MAXOBJ (F_VF_NUMERIC | 6)
#define F_VAR_PATH (F_VF_READONLY | 7)
#define F_VAR_EX_REFRESH (F_VF_NUMERIC | 8)
#define F_VAR_EX_EXPIRE (F_VF_NUMERIC | 9)
#define F_VAR_EX_EXPIRE_ERR (F_VF_NUMERIC | 10)
#define F_VAR_ATTR_FIRST (F_VF_READONLY | 256)
#define F_VAR_ATTR_LAST (F_VAR_ATTR_FIRST + 255)

struct filter_node {
  struct filter_node *next, *last;
  int op;
  union {
    struct {
      struct filter_cond *condition;
      struct filter_node *positive, *negative;
    } cond;
    char *msg;
    struct {
      uns var, val;
    } setnum;
  } u;
};

#define F_OP_ACCEPT 0
#define F_OP_REJECT 1
#define F_OP_LOG 2			/* (msg) */
#define F_OP_IF 3			/* (cond) */
#define F_OP_SET_NUM 4			/* (setnum) */
#define F_OP_INC_NUM 5			/* (setnum) */

struct filter {
  struct mempool *pool;
  struct filter_node *body;
};

/* parse.y */

int yyparse(void);

/* lex.l */

void filter_err(char *msg) NONRET;
void filter_lex_init(char *name);
void filter_lex_cleanup(void);
int yylex(void);

/* filter.c */

extern struct filter *filter_current;

struct filter_args {
  struct filter *filter;
  char *url;
  char *source;
  uns weight;
  uns tag;
  uns maxobj;
  uns ex_refresh, ex_expire, ex_expire_err;
  char *(*get_attr)(struct filter_args *a, int attr);
  void *priv;		/* Reserved for use by the caller */
  char *host;		/* Used internally */
  uns port;
  char *path;
  char buf[16];
};

struct filter *filter_load(char *);
struct filter *filter_load_default(void);
int filter_run(struct filter_args *);
