/*
 *	Sherlock Gatherer -- Configuration
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#include "gather.h"

#include <lib/url.h>

static byte *log_file;

struct distiller {
  node n;
  byte *patt;
  byte *name;
};

static list distillers;

static byte *
cf_downloader(struct cfitem *c, byte *l)
{
  byte *w[2];
  int proto;

  if (wordsplit(l, w, 2) != 2)
    return "Invalid downloader definition";
  if (! (proto = identify_protocol(w[0])))
    return "Unknown protocol";
  downloaders[proto] = stralloc(w[1]);
  return NULL;
}

static byte *
cf_distiller(struct cfitem *c, byte *l)
{
  byte *w[2];
  struct distiller *d = xmalloc(sizeof(struct distiller));

  if (wordsplit(l, w, 2) != 2)
    return "Invalid distiller definition";
  d->patt = stralloc(w[0]);
  d->name = stralloc(w[1]);
  add_tail(&distillers, &d->n);
  return NULL;
}

byte *
lookup_distiller(byte *type)
{
  struct distiller *d;

  DO_FOR_ALL(d, distillers)
    if (match_ct_patt(d->patt, type))
      return d->name;
  return NULL;
}

static struct cfitem cfitems[] = {
  { "LogFile", CI_STRING, &log_file },
  { "MaxThreads", CI_INT, &max_threads },
  { "MaxQueuedObjs", CI_INT, &max_queued_objs },
  { "TraceJobs", CI_INT, &trace_jobs },
  { "MinServerDelay", CI_INT, &min_server_delay },
  { "RobotFileTimeout", CI_INT, &robot_file_timeout },
  { "RecErrDelay1", CI_INT, &rec_err_dly1 },
  { "RecErrDelay2", CI_INT, &rec_err_dly2 },
  { "RecErrLimit", CI_INT, &rec_err_limit },
  { "DowngradeTimeout", CI_INT, &downgrade_timeout },
  { "TraceRefs", CI_INT, &trace_refs },
  { "RefreshAfter", CI_INT, &refresh_after },
  { "Downloader", CI_FUNCTION, cf_downloader },
  { "DoBackups", CI_INT, &dont_unlink },
  { "RobotName", CI_STRING, &robot_name },
  { "Distiller", CI_FUNCTION, cf_distiller },
  { "MaxQueueTime", CI_INT, &max_queue_time },
  { "MaxRecErr", CI_INT, &max_rec_err },
  { "AutoGoRoot", CI_INT, &auto_enqueue_root },
  { "ShowForbidden", CI_INT, &show_forbidden },
  { "IgnoreRefs", CI_INT, &ignore_refs },
  { "MaxObj", CI_INT, &max_obj_count },
  { "QueueCacheSize", CI_INT, &queue_cache_size },
  { NULL, CI_STOP, NULL }
};

void
read_config(void)
{
  init_list(&distillers);
  cf_read("cf/gatherd", cfitems);
  open_log_file(log_file);
}
