/*
 *	Sherlock Gatherer -- Declarations
 *
 *	(c) 1997--1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

#include <lib/lib.h>
#include <lib/lists.h>

/* main.c */

extern uns max_threads;
extern uns max_queued_objs;
extern uns trace_jobs, trace_refs;
extern uns min_server_delay;
extern uns robot_file_timeout, refresh_after;
extern uns rec_err_dly1, rec_err_dly2, rec_err_limit;
extern uns downgrade_timeout, dont_unlink, ignore_refs;
extern uns max_queue_time, max_rec_err, auto_enqueue_root, show_forbidden;
extern byte *downloaders[];

void maybe_unlink(byte *name);

#define BANNER "Sherlock Gatherer " SHER_VER

#define OID_NODATA 8888
#define OID_AUX 8889

/* db.c */

#define LOCK_NAME "db/GATHERLOCK"
#define URLDB_NAME "db/URL.db"
#define MD5DB_NAME "db/MD5.db"

void db_init(void);
void gather_lock(void);
void gather_unlock(void);

struct urlrec {
  u32 access;				/* Time of last access, bit0=queued */
  oid_t oid;				/* Object ID or error code if error */
};

void urldb_open(void);
void urldb_close(void);
int urldb_lookup(byte *, struct urlrec *);
void urldb_rewind(void);
byte *urldb_get_next(byte *, struct urlrec *);
int urldb_exists(byte *);
int urldb_delete(byte *);
void urldb_store(byte *, u32, oid_t);

struct md5rec {
  oid_t oid;
};

void md5db_open(void);
void md5db_close(void);
int md5db_lookup(byte *, struct md5rec *);
int md5db_exists(byte *);
void md5db_rewind(void);
byte *md5db_get_next(byte *, struct md5rec *);
int md5db_delete(byte *);
void md5db_store(byte *, oid_t);

/* queue.c */

#define HOST_HASH_SIZE 16384
#define HOST_FILE_NAME "db/hosts"
#define HOST_BACK_NAME "db/hosts.bak"

#define QUEUE_FILE_NAME "db/queue"
#define QUEUE_PAGE_SIZE 1024		/* Must be minimally URL_SIZE - 5 bytes for "xx://" + 4 bytes for next ptr */
#define QUEUE_PAGE_MASK (QUEUE_PAGE_SIZE-1)

extern uns queue_cache_size;

void queue_init(void);
void queue_cleanup(void);
void queue_sync(void);

struct qhost {				/* Representation of queued host */
  node n;
  struct qhost *hash;
  uns qf_pos;				/* Reference to first object in queue file */
  uns qf_last;				/* Reference to last object in queue file */
  oid_t host_id;			/* Host object ID */
  uns obj_count;			/* Number of objects known for this host */
  uns hf_pos;				/* Reference to qf_pos instance in host file */
  uns wait_until;			/* This object is waiting until specified time */
  word port;
  byte protocol;
  byte status;				/* Host status (QHF_xxx) */
  byte name[1];
};

#define QHF_NEW 0
#define QHF_QUEUED 1
#define QHF_ACTIVE 2
#define QHF_EMPTY 3
#define QHF_LOADED 4
#define QHF_ACTIVE_MODIFIED 5
#define QHF_WAITING 6

struct qitem {				/* Representation of queued data */
  uns aux;				/* Used internally */
  byte text[1];
};

struct qhost *dequeue_host(uns *, oid_t);
struct qhost *find_host(uns, byte *, uns);
void enqueue_host(struct qhost *, uns);
struct qhost *new_host(uns, byte *, uns, oid_t);
void wait_host(struct qhost *, uns);

struct qitem *dequeue_item(struct qhost *);
struct qitem *peek_item(struct qhost *);
void enqueue_item(struct qhost *, byte *);

/* host.c */

struct odes *load_host_obj(oid_t);
void dump_host_obj(oid_t, struct odes *);
oid_t new_host_id(void);

/* oqueue.c */

struct qobj {
  node n;
  struct odes *obj;			/* Used for passing cached odes between bg and fg jobs */
  struct tempfile tf;			/* Temporary file holding object data */
};

extern uns qobj_count;

void oqueue_init(void);
struct qobj *qobj_new(void);		/* -> open temp file, not queued */
void qobj_del(struct qobj *);		/* Discard */
void qobj_enqueue(struct qobj *);
struct qobj *qobj_enqueue_name(byte *);
struct qobj *qobj_dequeue(void);

/* config.c */

void read_config(void);
byte *lookup_distiller(byte *);

/* url_filter.c */

extern byte *robot_name;
extern int trace_filter, trace_robots, allow_relative, trace_subst, max_obj_count;

struct url;
struct filter_args;

void url_filter_init(void);
uns okay_to_get(struct odes *, struct url *, struct filter_args *);
void parse_robot_file(struct odes *, struct odes *);
int translate_url(struct url *, byte *, byte *, byte *);

/* datatype.c */

extern int trace_data_types;
extern int dt_fh;
extern uns dt_in_size, dt_where;
extern byte dt_inbuf[];

void datatype_init(void);
int datatype_guess(struct odes *);		/* 0=OK, <0=call decoder, else error */
int datatype_decode(struct odes *, int);	/* 0=OK, else error */
int dt_read_head(void);
int dt_read_next(void);

/* meta.c */

extern uns meta_start, meta_end;

int hunt_meta(byte *, byte *);

/* decode.c */

extern uns max_decode;

int c_e_decode(struct odes *, int);

#define ET_GZIP -1
#define ET_DEFLATE -2

/* charset.c */

extern int trace_charsets;

void charset_init(void);
int guess_charset(struct odes *, byte *, byte *);
void char_convert_init(struct odes *);
void char_convert(byte **, uns *, byte **, uns *);
