/*
 *	Sherlock Gatherer -- Searching for HTML <META> Hacks
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <string.h>

#include "gather.h"

#include <lib/string.h>

uns meta_start, meta_end;

#define STATE(x) x: switch (*s++) {
#define IGNORE {			/* Keep Emacs autoindent less confused */
#define GO(x,y) case x: goto y;
#define END(x) case 0: return 0; default: goto x; }
#define UEND(x) case 0: return 0; default: s--; goto x; }

int					/* Very ugly part -- expecting we know a little about HTML syntax :-( */
hunt_meta(byte *buf, byte *dest)	/* Let's implement it as a state machine */
{
  byte *s = buf;			/* Buffer scan pointer */
  byte *p;				/* Current meta: start */
  byte *attrname0, *attrname1;		/* Start and end of attribute name */
  byte *attrval0, *attrval1;		/* Start and end of attribute value */
  byte *httpequiv0, *httpequiv1;	/* Http-Equiv attribute */
  byte *contents0, *contents1;		/* Content attribute */

  STATE(normal)				/* Default context */
    GO('<', left)
  END(normal)

  STATE(left)				/* "<" */
    GO('m', leftm)
    GO('M', leftm)
    GO('!', maycmt)
    GO('\r', normal)
    GO('\n', normal)
  UEND(inside)

  STATE(maycmt)
    GO('-', maycmt2)
  UEND(inside)

  STATE(maycmt2)			/* "<!-" */
    GO('-', comment)
  UEND(inside)

  STATE(comment)			/* "<!--" | inside comment */
    GO('-', comm2)
  END(comment)

  STATE(comm2)				/* comment ... "-" */
    GO('-', comm3)
  END(comment)

  STATE(comm3)				/* comment ... "--" */
    GO('>', normal)
    GO('-', comm3)
  END(comment)

  STATE(leftm)				/* "<M" */
    GO('e', leftme)
    GO('E', leftme)
  UEND(inside)

  STATE(leftme)				/* "<ME" */
    GO('t', leftmet)
    GO('T', leftmet)
  UEND(inside)

  STATE(leftmet)			/* "<MET" */
    GO('a', meta)
    GO('A', meta)
  UEND(inside)

  STATE(inside)				/* "<" ... */
    GO('>', normal)
    GO('"', quote)
  END(inside)

  STATE(quote)				/* "<" ... quoted ... */
    GO('\r', normal)			/* Quoting error */
    GO('\n', normal)
    GO('"', inside)
  END(quote)

meta:					/* "<META" */
  p = s - 5;
  httpequiv0 = httpequiv1 = NULL;
  contents0 = contents1 = NULL;
insmeta:				/* "<META" ... */
  switch (*s++)
    {
    case '>':
      if (httpequiv0 && contents0 && !strncasecmp(httpequiv0, "Content-Type", httpequiv1 - httpequiv0))
	{
	  meta_start = p - buf;
	  meta_end = s - buf;
	  if (meta_end - meta_start > 100)
	    {
	      log(L_ERROR "META Content-Type too long");
	      return 0;
	    }
	  memcpy(dest, contents0, contents1 - contents0);
	  dest[contents1 - contents0] = 0;
	  return 1;
	}
      goto normal;
    case 0:
      return 0;
    default:
      if (Calpha(s[-1]))
	{
	  attrname0 = s-1;
	  goto metaattr;
	}
      else
	goto insmeta;
    }

metaattr:				/* "<META" ... attribute */
  if (*s == '>' || Cspace(*s))
    goto insmeta;
  else if (!*s)
    return 0;
  else if (*s++ == '=')
    {
      attrname1 = s-1;
      attrval0 = s;
      if (*s == '"')
	{
	  attrval0 = ++s;
	  goto quoteval;
	}
      else
	goto metaval;
    }
  else
    goto metaattr;

  STATE(metaval)			/* "<META" ... attribute "=" value */
    GO('>', endval)
    GO(' ', endval)
    GO('\t', endval)
    GO('\r', endval)
    GO('\n', endval)
  END(metaval)

  STATE(quoteval)			/* "<META" ... attribute "=" quoted_value */
    GO('"', endval)
    GO('\r', normal)			/* Quoting error */
    GO('\n', normal)
  END(quoteval)

endval:
  attrval1 = s-1;
  if (!strncasecmp(attrname0, "HTTP-Equiv", attrname1 - attrname0))
    {
      httpequiv0 = attrval0;
      httpequiv1 = attrval1;
    }
  else if (!strncasecmp(attrname0, "Contents", attrname1 - attrname0))
    {
      contents0 = attrval0;
      contents1 = attrval1;
    }
  goto insmeta;
}
