/*
 *	Sherlock Library -- Object Names
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>

#include "lib.h"

oid_t
new_oid(uns retry)
{
  static oid_t last_oid;
  static uns retry1, retry2;

  if (last_oid < OID_MIN || retry > 16)
    {
    rand:
      retry1 = retry2 = 0;
      do
        last_oid = temprand(retry);
      while (last_oid < OID_MIN);
      return last_oid;
    }
retry:
  if (retry2++ < 15)
    last_oid = (last_oid & ~0xffff) | (random() & 0xffff);
  else
    {
      retry2 = 0;
      if (retry1++ < 3)
        last_oid = (last_oid & ~0xffffff) | (random() & 0xffffff);
      else
        goto rand;
    }
  if (last_oid < OID_MIN)
    goto retry;
  return last_oid;
}

void
mk_obj_name(byte *dir, oid_t id, byte *buf)
{
  sprintf(buf, "%s/%02x/%02x/%04x", dir, (id >> 24U) & 0xff, (id >> 16U) & 0xff, id & 0xffff);
}

static int
open_obj_file(byte *dir, oid_t id, int unique)
{
  byte buf[32];
  byte *d;
  int fd;
  int flags = O_CREAT | O_WRONLY | (unique ? O_EXCL : O_TRUNC);
  int om = unique ? 0600 : 0666;

  mk_obj_name(dir, id, buf);

  if ((fd = open(buf, flags, om)) >= 0)
    return fd;
  if (errno == EEXIST)
    return -1;

  d = buf + strlen(dir);
  d[6] = 0;
  if (mkdir(buf, 0777) < 0 && errno != EEXIST)
    {
      d[3] = 0;
      if (mkdir(buf, 0777) < 0 && errno != EEXIST)
	goto fail;
      d[3] = '/';
      if (mkdir(buf, 0777) < 0 && errno != EEXIST)
	goto fail;
    }
  d[6] = '/';

  if ((fd = open(buf, flags, om)) >= 0)
    return fd;

fail:
  mk_obj_name(dir, id, buf);
  die("Unable to create object %s: %m", buf);
}

int
dump_obj_to_file(byte *dir, oid_t id, struct odes *d, int unique)
{
  int fd;
  FILE *f;

  if ((fd = open_obj_file(dir, id, unique)) < 0)
    return 0;
  if (! (f = fdopen(fd, "w")))
    die("dump_obj_to_file: fdopen: %m");
  obj_fwrite(f, d);
  fflush(f);
  if (unique)
    fchmod(fd, 0644);
  fclose(f);
  return 1;
}
