/*
 *	Sherlock Library -- Structure of Search Engine Databases
 *
 *	(c) 1997--1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

/* Structure of each database item: */

struct sdbitem {
  oid_t oid;				/* Object ID */
  byte tag;				/* Section tag */
  byte flags;				/* High 4 bits: word type */
					/* Low 4 bits: word quality (-1 to 14) */
#ifdef SHERLOCK_CONFIG_REF_WEIGHTS
  byte weight;				/* Additional weight assigned to this ref */
#define WW_MAX 255
#define WW_OFFSET 128
#endif
} __attribute__((packed));

#define MAX_TAG 256
#define ANY_TAG 0xff

#define WT_SHIFT 4
#define WQ_MASK 0x0f
#define WT_MASK 0xf0
#define WQ_OFFSET 1

#define WORDTYPE(flags) (((flags) & WT_MASK) >> WT_SHIFT)

/* Known word types (update search/daemon/lex.c when changing!) */

#define WT_WORD 0			/* Normal text word */
#define WT_REF 1			/* URL Reference */
#define WT_REF_IMG 2			/* URL Reference to image */
#define WT_URL 3			/* URL Self-Reference */
#define WT_HOST 4			/* Host name */
#define WT_DOMAIN 5			/* Domain name */
#define WT_KEYWORD 6			/* Raw string in non-word list */

/* Maximal length of word */

#define MAX_WORD_LEN 1024		/* URL should fit here */

/* Length of file page */

#define FILE_PAGE_SIZE 4096

/* Signature record in the SBTree */

struct sig {
  byte sig[8];
  u32 ptr;				/* Word list is hopefully small */
};

/* Parameters of the SBTree */

#define TREE_PAGE_SIZE 1024
#define TREE_PAGE_DATA (TREE_PAGE_SIZE - sizeof(u32))
#define KEYS_PER_PAGE (TREE_PAGE_DATA / sizeof(struct sig))
#define PAD_PER_PAGE (TREE_PAGE_DATA - sizeof(struct sig)*KEYS_PER_PAGE)

struct tree_page {
  struct sig keys[KEYS_PER_PAGE];	/* The keys */
  u32 aux;				/* Auxilliary data: for root page: total # of keys in the tree */
  byte pad[PAD_PER_PAGE];		/* Padding to end of the page */
};

/* Non-word hashing */

#define HASH_MAGIC 23
