/*
 *	Sherlock Search Engine -- Administration Commands
 *
 *	(c) 1997--1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "sherlock.h"

static void
cmd_stat(int argc, byte **argv)
{
  int cnt, i;
  struct database *db;

  add_qr("+OK");
  add_qr("T%d", (int) time(NULL));
  cnt = 0;
  for(db=databases; db; db=db->next)
    cnt++;
  add_qr("N%d", cnt);
  add_qr("");
  for(db=databases; db; db=db->next)
    {
      add_qr("D%s", db->obj_pool_name);
      add_qr("T%d", (int) db->last_modified);
      for(i=0; i<256; i++)
	{
	  if (db->section_count[i])
	    add_qr("S%d %d", i, db->section_count[i]);
	}
      add_qr("O%d", (int) db->num_ids);
      add_qr("");
    }
}

struct command {
  char *name;
  void (*handler)(int argc, byte **argv);
  int argmin, argmax, passwd;
};

static struct command cmds[] = {
  { "stats",	cmd_stat,	0, 0, 0 },
  { NULL,	NULL,		0, 0, 0 }
};

void
do_command(struct query *q)
{
  byte *c = q->cmd;
  byte *words[16], **argv;
  int argc;
  struct command *cmd = cmds;

  argc = wordsplit(c, words, sizeof(words)/sizeof(byte *));
  if (argc <= 0)
    {
      add_qr("-109 Malformed command");
      return;
    }
  argv = words;
  argc--;
  while (cmd->name)
    {
      if (!strcasecmp(cmd->name, argv[0]))
	{
	  argv++;
	  if (cmd->passwd)
	    {
	      if (!argc || strcmp(control_password, argv[0]))
		{
		  add_qr("-109 Invalid control password");
		  log(L_WARN "`%s' used instead of control password", argv[0]);
		  return;
		}
	      argc--, argv++;
	    }
	  if (argc < cmd->argmin || argc > cmd->argmax)
	    {
	      add_qr("-109 Invalid number of parameters");
	      return;
	    }
	  cmd->handler(argc, argv);
	  return;
	}
      cmd++;
    }
  add_qr("-109 Unknown control command");
}
