/*
 *	Sherlock Search Engine -- Searching for Non-Words
 *
 *	(c) 1997--1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "sherlock.h"

#include <lib/url.h>
#include <lib/pools.h>

static inline uns
nw_hash(byte *w, uns cnt)
{
  uns i = 0;
  do
    i = ((i * HASH_MAGIC) + *w) % cnt;
  while (*w++);
  return i;
}

int
lookup_nonword(struct nonword_index *ix, struct known_word *kw)
{
  byte *w = kw->text;
  sh_off_t *x = kw->refs;
  uns h = nw_hash(w, ix->hash_val);
  uns l, s;
  sh_off_t *ppos, pos, ref;
  byte buf[MAX_WORD_LEN], *b, *r, *re;

  ppos = (sh_off_t *) read_data(ix->hash_fd, h * sizeof(sh_off_t), &l);
  if (l < sizeof(sh_off_t))
    die("lookup_nonword(): Lost sword... Awaiting doom...");
  pos = *ppos;
  DBG("%s: hash %d, pos %d", w, h, pos);
  if (pos == (sh_off_t) -1)
    return 0;
  b = buf;
  s = 0;
  for(;;)
    {
      r = read_data(ix->list_fd, pos, &l);
      re = r + l;
      pos += l;
      while (r < re)
	{
	  if (s)
	    {
	      *b++ = *r++;
	      if (!--s)
		{
		  if (!strcmp(buf, w))
		    {
		      *x = ref;
		      return 1;
		    }
		  b = buf;
		}
	    }
	  else if (*r)
	    *b++ = *r++;
	  else
	    {
	      *b = *r++;
	      if (b == buf)
		return 0;
	      ref = 0;
#ifdef CPU_BIG_ENDIAN
	      b = (byte *) &ref + sizeof(sh_off_t) - BYTES_PER_FILE_POINTER;
#else
	      b = (byte *) &ref;
#endif
	      s = BYTES_PER_FILE_POINTER;
	    }
	}
    }
  return 0;
}

static int
nonword_lookup(struct index *index, struct known_word *kw)
{
  if (!kw->is_nonword)
    return 0;
  return lookup_nonword((struct nonword_index *) index, kw);
}

void
nonword_init(struct index *index)
{
  struct nonword_index *idx = (struct nonword_index *) index;

  idx->hash_fd = open_file(idx->hash_name);
  idx->list_fd = open_file(idx->list_name);
  idx->hash_val = lseek(idx->hash_fd, 0, SEEK_END) / sizeof(sh_off_t);
  DBG("Non-word list: %d buckets", idx->hash_val);
  index->lookup = nonword_lookup;
}

byte *
normalize_nonword(struct mempool *p, byte *a, uns classmap)
{
  byte b1[MAX_URL_SIZE], b2[MAX_URL_SIZE], b3[MAX_URL_SIZE], b4[MAX_URL_SIZE];
  int e;
  struct url url;

  if (classmap & ((1 << WT_WORD) | (1 << WT_KEYWORD)))
    return a;

  if (classmap & ((1 << WT_HOST) | (1 << WT_DOMAIN)))
    {
      strcpy(b1, a);
      bzero(&url, sizeof(url));
      url.host = b1;
      if (e = url_canonicalize(&url))
	goto urlerr;
      a = pool_alloc(p, 1 + strlen(url.host));
      strcpy(a, url.host);
      return a;
    }

  if (   (e = url_canon_split(a, b1, b2, &url))
      || (e = url_pack(&url, b3))
      || (e = url_enescape(b3, b4)))
    goto urlerr;
  a = pool_alloc(p, 1 + strlen(b4));
  strcpy(a, b4);
  return a;

urlerr:
  sprintf(b1, "-%d %s", 200+e, url_error(e));
  err(b1);
}
