/*
 *	Sherlock Search Engine -- Query Processing
 *
 *	(c) 1997--1999 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "sherlock.h"

#include <lib/lfs.h>

void
show_object(struct query *q, oid_t oid)
{
  uns len, l;
  sh_off_t *z, pos;
  struct database *db;

  for(db=databases; db; db=db->next)
    if (oid >= db->first_id && oid < db->first_id + db->num_ids)
      break;
  if (!db)
    {
      add_qr("!Invalid object ID %08x", oid);
      return;
    }
  oid -= db->first_id;
  z = (sh_off_t *) read_data(db->obj_idx_fd, oid * sizeof(sh_off_t), &l);
  if (l < sizeof(sh_off_t))
    die("Ugly sh_off_t misalign!");
  pos = *z;
  z = (sh_off_t *) read_data(db->obj_idx_fd, (oid+1) * sizeof(sh_off_t), &l);
  if (l < sizeof(sh_off_t))
    die("Ugly sh_off_t misalign!");
  len = *z - pos - 1;
  DDBG("Object %08x at %08x, len=%d", oid, pos, len);

  while (len)
    {
      byte *b = read_data(db->obj_pool_fd, pos, &l);
      if (l > len)
	l = len;
      reply_string(q, b, l);
      pos += l;
      len -= l;
    }
}

void
objects_init(void)
{
  struct database *db;
  oid_t oid = 0;
  int i;

  for(db=databases; db; db=db->next)
    {
      db->obj_pool_fd = open_file(db->obj_pool_name);
      db->obj_idx_fd = open_file(db->obj_idx_name);
      db->num_ids = sh_seek(db->obj_idx_fd, 0, SEEK_END) / sizeof(sh_foff_t);
      db->first_id = oid;
      oid += db->num_ids;
      log("%s: %d objects (ID %08x to %08x)", db->obj_pool_name, db->num_ids, db->first_id, oid-1);
      sh_seek(db->obj_pool_fd, -(sh_off_t)sizeof(db->section_count), SEEK_END);
      if (read(db->obj_pool_fd, db->section_count, sizeof(db->section_count)) != sizeof(db->section_count))
	die("Error reading section counts from %s: %m", db->obj_pool_name);
      for(i=0; i<256; i++)
	if (db->section_count[i])
	  db->section_flags[i >> 3] |= 1 << (i & 7);
    }
}
