/*
 *	Trivial Cross-Reference Generator
 *
 *	(c) 1998 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include <lib/lib.h>
#include <lib/fastbuf.h>
#include <lib/string.h>
#include <lib/searchdb.h>

static struct fastbuf *dest_list, *dest_idx, *dest_words;

static uns lino = 0;

/* Shamelessly stolen from objget.c */

static uns cache_size = 1048576;

struct cword {
  struct cword *next;
  uns ref;
  word len;
  byte w[1];
};

static byte *word_cache, *wc_ptr, *wc_end;
static struct cword *first_cword;

static void
reset_word_cache(void)
{
  first_cword = NULL;
  wc_ptr = word_cache;
}

static int
wc_cmp(struct cword *A, struct cword *B)
{
  int res;

  res = strcmp(A->w, B->w);
  if (!res)
    {
      if (A->ref < B->ref)
	return -1;
      if (A->ref > B->ref)
	return 1;
    }
  return res;
}

static struct cword *
sort_word_list(struct cword *x)
{
  struct cword *l, **ll, *r, **rr, *p;

  if (!x || !x->next)			/* Trivial cases */
    return x;

  ll = &l;				/* Split */
  rr = &r;
  while (x)
    {
      *ll = x;
      ll = &x->next;
      x = x->next;
      if (!x)
	break;
      *rr = x;
      rr = &x->next;
      x = x->next;
    }
  *ll = NULL;
  *rr = NULL;

  l = sort_word_list(l);		/* Sort */
  r = sort_word_list(r);

  ll = &x;				/* Merge */
  while (l && r)
    {
      if (wc_cmp(l, r) < 0)
	{
	  p = l;
	  l = l->next;
	}
      else
	{
	  p = r;
	  r = r->next;
	}
      *ll = p;
      ll = &p->next;
    }
  if (l)
    *ll = l;
  else
    *ll = r;
  return x;
}

static void
flush_word_list(void)
{
  struct cword *cw, *tw;
  uns n, old;

  if (!first_cword)
    return;
  cw = sort_word_list(first_cword);
  while (cw)
    {
      n = 1;
      old = cw->ref;
      tw = cw;
      cw = cw->next;
      while (   cw
	     && cw->len == tw->len
	     && !strcmp(cw->w, tw->w))
	{
	  if (cw->ref != old)
	    {
	      n++;
	      old = cw->ref;
	    }
	  cw = cw->next;
	}
      bwrite(dest_words, tw->w, tw->len+1);
      bputl(dest_words, n);
      old = ~0;
      while (tw != cw)
	{
	  if (tw->ref != old)
	    {
	      struct sdbitem obj;
	      old = tw->ref;
	      bzero(&obj, sizeof(obj));
	      obj.oid = tw->ref;
	      obj.flags = (WT_KEYWORD << WT_SHIFT) | WQ_OFFSET;
	      bwrite(dest_words, &obj, sizeof(obj));
	    }
	  tw = tw->next;
	}
    }
}

static inline void
do_word(byte *w)
{
  struct cword *wc;
  uns x;
  uns l = strlen(w);

#if 0
  if (l < 3)
    return;
#endif

  x = (sizeof(struct cword) + l + CPU_STRUCT_ALIGN - 1) & ~(CPU_STRUCT_ALIGN - 1); /* Magic */
  if (wc_ptr + x > wc_end)
    {
      flush_word_list();
      reset_word_cache();
    }
  wc = (struct cword *) wc_ptr;
  wc->ref = lino;
  memcpy(wc->w, w, l);
  wc->w[l] = 0;
  wc->len = l;

#if 0
  for(weed = weed_hash[weed_hash_val(wc->w)]; weed; weed=weed->next)
    if ((uns) weed->len == l && !strcmp(weed->w, wc->w))
      return;
#endif

  wc_ptr += x;
  wc->next = first_cword;
  first_cword = wc;
}

static void
init_word_cache(void)
{
  word_cache = xmalloc(cache_size);
  wc_end = word_cache + cache_size;
  reset_word_cache();
}

static void
do_it(byte *p, byte *stop)
{
  while (p < stop)
    {
      uns c = *p++;
      if (Ccat(c, _C_WSTART))
	{
	  byte sbuf[256];
	  byte *d = sbuf;
	  byte *e = sbuf + sizeof(sbuf) - 2;
	  *d++ = c;
	  while (p < stop && Cword(c = *p))
	    {
	      p++;
	      if (d >= e)
		die("Word too long");
	      *d++ = c;
	    }
	  *d = 0;
	  do_word(sbuf);
	}
    }
}

static void
do_file(char *name)
{
  int fd;
  struct stat st;
  void *at;

  fd = open(name, O_RDONLY);
  if (fd < 0)
    die("Unable to open %s: %m", name);
  if (fstat(fd, &st) < 0)
    die("stat(%s): %m", name);
  if (!(at = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0)))
    die("mmap(%s): %m", name);

  bputp(dest_idx, btell(dest_list));
  bputs(dest_list, "Ufile:/");
  bputsn(dest_list, name);
  bputc(dest_list, 0);

  printf("+++ %d: %s\n", lino, name);
  do_it(at, at+st.st_size);
  lino++;

  munmap(at, st.st_size);
  close(fd);
}

int
main(int argc, char **argv)
{
  char name[1024];

  if (argc != 4)
    {
      fprintf(stderr, "Usage: txref <dest-list> <dest-index> <dest-words>\n");
      return 1;
    }
  initlog(argv[0]);
  dest_list = bopen(argv[1], O_WRONLY|O_CREAT|O_TRUNC, 65536);
  dest_idx = bopen(argv[2], O_WRONLY|O_CREAT|O_TRUNC, 65536);
  dest_words = bopen(argv[3], O_WRONLY|O_CREAT|O_TRUNC, 65536);
  init_word_cache();

  while (fgets(name, sizeof(name), stdin))
    {
      char *c = strchr(name, '\n');
      if (!c)
	die("File name too long");
      *c = 0;
      do_file(name);
    }

  flush_word_list();
  bputp(dest_idx, btell(dest_list));
  bclose(dest_list);
  bclose(dest_idx);
  bclose(dest_words);
  return 0;
}
